/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package rs.ac.bg.fon.silab.gui.example1.components.table.model;

import java.util.List;
import javax.swing.table.AbstractTableModel;
import rs.ac.bg.fon.silab.jdbc.example1.domen.BillEntity;
import rs.ac.bg.fon.silab.jdbc.example1.domen.BillItemEntity;
import rs.ac.bg.fon.silab.jdbc.example1.domen.CompanyEntity;
import rs.ac.bg.fon.silab.jdbc.example1.domen.ProductEntity;

/**
 *
 * @author FON
 */
public class BillTableModel extends AbstractTableModel {

    private final BillEntity bill;
    private String[] columnNames = new String[]{"Item", "Product", "Quantity", "Amount", "Measure unit"};

    public BillTableModel(BillEntity bill) {
        this.bill = bill;
    }

    @Override
    public int getRowCount() {
        if (bill == null || bill.getItems() == null) {
            return 0;
        }
        return bill.getItems().size();
    }

    @Override
    public int getColumnCount() {
        return columnNames.length;
    }

    @Override
    public Object getValueAt(int rowIndex, int columnIndex) {
        BillItemEntity billItem = bill.getItems().get(rowIndex);
        switch (columnIndex) {
            case 0:
                return billItem.getItemId();
            case 1:
                return billItem.getProductEntity().getName();
            case 2:
                return billItem.getQuantity();
            case 3:
                return billItem.getAmount();
            case 4:
                return billItem.getUnitMeasure();
            default:
                return "N/A";
        }
    }

    @Override
    public void setValueAt(Object value, int rowIndex, int columnIndex) {
        BillItemEntity item = bill.getItems().get(rowIndex);
        if (columnIndex == 1) {
            item.setProductEntity((ProductEntity) value);
            setAmount(item);
        }
        if (columnIndex == 2) {
            item.setQuantity(Double.parseDouble(value.toString()));
            setAmount(item);
        }

    }

    @Override
    public boolean isCellEditable(int rowIndex, int columnIndex) {
        return (columnIndex == 1 || columnIndex == 2);
    }

    @Override
    public String getColumnName(int column) {
        return columnNames[column];
    }

    public void addRow() {
        //validate values in bill items
        if (bill.getItems().size() == 0 
                || (bill.getItems().get(bill.getItems().size() -1).getProductEntity().getId() != null
                && bill.getItems().get(bill.getItems().size() -1).getQuantity() > 0)) {
            BillItemEntity billItem = new BillItemEntity();
            billItem.setItemId(bill.getItems().size() + 1);
            bill.getItems().add(billItem);

            fireTableDataChanged();
        }

    }

    private void setAmount(BillItemEntity item) {
        if (item.getProductEntity() != null && item.getQuantity() != null && item.getQuantity() > 0) {
            item.setAmount(item.getQuantity() * item.getProductEntity().getPrice());
            Double billAmount = 0.0;
            for (BillItemEntity billItem : bill.getItems()) {
                billAmount += billItem.getAmount();
            }
            bill.setAmount(billAmount);
        }
    }

    public BillEntity getBill() {
        return bill;
    }

    public void removeRow(int selectedRow) {
        if (selectedRow >= 0 && selectedRow < bill.getItems().size()) {
            bill.getItems().remove(selectedRow);
            updateBill();
            fireTableDataChanged();
        }
    }

    private void updateBill() {
        int item = 0;
        Double billAmount = 0.0;
        for (BillItemEntity billItem : bill.getItems()) {
            item++;
            billItem.setItemId(item);
            billAmount += billItem.getAmount();
        }
        bill.setAmount(billAmount);
    }

}
